@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-import gtk 
+import gtk
 
 def change_callback(self, window, entry):
 	print("text=%s" % entry.get_text())
@@ -26,8 +26,5 @@ def main():
 	return 0
 
 if __name__ == "__main__":
-	try:
-		main()
-	except Exception, e:
-		print("error: %s" % e)
+	main()
 
