@@ -8,6 +8,7 @@
 Everyone else should just use prop_set/prop_get with nice clean Python calling
 conventions, and if you need more (un)marshalling smarts, add them here."""
 
+import traceback
 import struct
 try:
     from StringIO import StringIO   #@UnresolvedImport @UnusedImport
@@ -25,6 +26,7 @@ from wimpiggy.lowlevel import (
                 const,                      #@UnresolvedImport
                 premultiply_argb_in_place   #@UnresolvedImport
                )
+from wimpiggy.xsettings_prop import set_settings, get_settings
 from wimpiggy.error import trap, XError
 from wimpiggy.log import Logger
 log = Logger()
@@ -108,7 +110,7 @@ class WMHints(object):
         else:
             self.start_iconic = None
         if flags & const["InputHint"]:
-            self.input = input
+            self.input = _input
         else:
             self.input = None
 
@@ -133,8 +135,8 @@ def _read_image(disp, stream):
         if not header:
             return None
         (width, height) = struct.unpack("=II", header)
-        bytes = stream.read(width * height * 4)
-        if len(bytes) < width * height * 4:
+        data = stream.read(width * height * 4)
+        if len(data) < width * height * 4:
             log.warn("Corrupt _NET_WM_ICON")
             return None
     except Exception, e:
@@ -152,7 +154,7 @@ def _read_image(disp, stream):
         log.warn("Your Cairo is too old! Carrying on as best I can, "
                  "but don't expect a miracle")
         return None
-    surf.get_data()[:] = bytes
+    surf.get_data()[:] = data
     # Cairo uses premultiplied alpha. EWMH actually doesn't specify what it
     # uses, but apparently the de-facto standard is non-premultiplied. (At
     # least that's what Compiz's sources say.)
@@ -175,16 +177,40 @@ def NetWMIcons(disp, data):
     return icons[-1][1]
 
 def _get_atom(disp, d):
-    unpacked = struct.unpack("=I", d)[0]
+    unpacked = struct.unpack("@I", d)[0]
     pyatom = get_pyatom(disp, unpacked)
     if not pyatom:
         log.error("invalid atom: %s - %s", repr(d), repr(unpacked))
         return  None
     return str(pyatom)
 
+def _get_multiple(disp, d):
+    uint_struct = struct.Struct("@I")
+    log("get_multiple struct size=%s, len(%s)=%s", uint_struct.size, d, len(d))
+    if len(d)!=uint_struct.size and False:
+        log.info("get_multiple value is not an atom: %s", d)
+        return  str(d)
+    return _get_atom(disp, d)
+
+
+def set_xsettings_format(use_tuple=True):
+    log("set_xsettings_format(%s)", use_tuple)
+    global _prop_types
+    if use_tuple:
+        _prop_types["xsettings-settings"] = (tuple, "_XSETTINGS_SETTINGS", 8,
+                           set_settings,
+                           get_settings,
+                           None)
+    else:
+        #for old clients that rely on the old raw string format:
+        _prop_types["xsettings-settings"] = (str, "_XSETTINGS_SETTINGS", 8,
+                           lambda disp, c: c,
+                           lambda disp, d: d,
+                           None)
+
 
 _prop_types = {
-    # Python type, X type Atom, format, serializer, deserializer, list
+    # Python type, X type Atom, formatbits, serializer, deserializer, list
     # terminator
     "utf8": (unicode, "UTF8_STRING", 8,
              lambda disp, u: u.encode("UTF-8"),
@@ -198,7 +224,7 @@ _prop_types = {
                lambda disp, d: d.decode("latin1"),
                "\0"),
     "atom": (str, "ATOM", 32,
-             lambda disp, a: struct.pack("=I", get_xatom(a)),
+             lambda disp, a: struct.pack("@I", get_xatom(a)),
               _get_atom,
              ""),
     "u32": ((int, long), "CARDINAL", 32,
@@ -223,9 +249,9 @@ _prop_types = {
                       unsupported, NetWMStrut, None),
     "icon": (cairo.ImageSurface, "CARDINAL", 32,
              unsupported, NetWMIcons, None),
-    "xsettings-settings": (str, "_XSETTINGS_SETTINGS", 8,
-                           lambda disp, d: d,
-                           lambda disp, d: d,
+    "xsettings-settings": (tuple, "_XSETTINGS_SETTINGS", 8,
+                           set_settings,
+                           get_settings,
                            None),
     # For uploading ad-hoc instances of the above complex structures to the
     # server, so we can test reading them out again:
@@ -238,85 +264,86 @@ _prop_types = {
     # specified what 'type' one should use; you just fetch with
     # AnyPropertyType and assume that what you get is a bunch of pairs of
     # atoms.
-    "multiple-conversion": (str, 0, 32, unsupported, _get_atom, None),
+    "multiple-conversion": (str, 0, 32, unsupported, _get_multiple, None),
     }
 
-def _prop_encode(disp, type, value):
-    if isinstance(type, list):
-        return _prop_encode_list(disp, type[0], value)
+def _prop_encode(disp, etype, value):
+    if isinstance(etype, list):
+        return _prop_encode_list(disp, etype[0], value)
     else:
-        return _prop_encode_scalar(disp, type, value)
+        return _prop_encode_scalar(disp, etype, value)
 
-def _prop_encode_scalar(disp, type, value):
-    (pytype, atom, format, serialize, deserialize, terminator) = _prop_types[type]
+def _prop_encode_scalar(disp, etype, value):
+    (pytype, atom, formatbits, serialize, _, _) = _prop_types[etype]
     assert isinstance(value, pytype), "value for atom %s is not a %s: %s" % (atom, pytype, type(value))
-    return (atom, format, serialize(disp, value))
+    return (atom, formatbits, serialize(disp, value))
 
-def _prop_encode_list(disp, type, value):
-    (pytype, atom, format, serialize, deserialize, terminator) = _prop_types[type]
+def _prop_encode_list(disp, etype, value):
+    (_, atom, formatbits, _, _, terminator) = _prop_types[etype]
     value = list(value)
-    serialized = [_prop_encode_scalar(disp, type, v)[2] for v in value]
+    serialized = [_prop_encode_scalar(disp, etype, v)[2] for v in value]
     no_none = [x for x in serialized if x is not None]
     # Strings in X really are null-separated, not null-terminated (ICCCM
     # 2.7.1, see also note in 4.1.2.5)
-    return (atom, format, terminator.join(no_none))
+    return (atom, formatbits, terminator.join(no_none))
 
 
-def prop_set(target, key, type, value):
+def prop_set(target, key, etype, value):
     trap.call_unsynced(XChangeProperty, target, key,
-                       _prop_encode(target, type, value))
+                       _prop_encode(target, etype, value))
 
-def _prop_decode(disp, type, data):
-    if isinstance(type, list):
-        return _prop_decode_list(disp, type[0], data)
+def _prop_decode(disp, etype, data):
+    if isinstance(etype, list):
+        return _prop_decode_list(disp, etype[0], data)
     else:
-        return _prop_decode_scalar(disp, type, data)
+        return _prop_decode_scalar(disp, etype, data)
 
-def _prop_decode_scalar(disp, type, data):
-    (pytype, atom, format, serialize, deserialize, terminator) = _prop_types[type]
+def _prop_decode_scalar(disp, etype, data):
+    (pytype, _, _, _, deserialize, _) = _prop_types[etype]
     value = deserialize(disp, data)
-    assert value is None or isinstance(value, pytype)
+    assert value is None or isinstance(value, pytype), "expected a %s but value is a %s" % (pytype, type(value))
     return value
 
-def _prop_decode_list(disp, type, data):
-    (pytype, atom, format, serialize, deserialize, terminator) = _prop_types[type]
+def _prop_decode_list(disp, etype, data):
+    (_, _, formatbits, _, _, terminator) = _prop_types[etype]
     if terminator:
         datums = data.split(terminator)
     else:
         datums = []
-        nbytes = format // 8
+        nbytes = formatbits // 8
         while data:
             datums.append(data[:nbytes])
             data = data[nbytes:]
-    props = [_prop_decode_scalar(disp, type, datum) for datum in datums]
+    props = [_prop_decode_scalar(disp, etype, datum) for datum in datums]
     #assert None not in props
     return [x for x in props if x is not None]
 
 # May return None.
-def prop_get(target, key, type, ignore_errors=False):
-    if isinstance(type, list):
-        scalar_type = type[0]
+def prop_get(target, key, etype, ignore_errors=False):
+    if isinstance(etype, list):
+        scalar_type = etype[0]
     else:
-        scalar_type = type
-    (pytype, atom, format, serialize, deserialize, terminator) = _prop_types[scalar_type]
+        scalar_type = etype
+    (_, atom, _, _, _, _) = _prop_types[scalar_type]
     try:
         #print(atom)
         data = trap.call_synced(XGetWindowProperty, target, key, atom)
         #print(atom, repr(data[:100]))
     except NoSuchProperty:
-        log.debug("Missing property %s (%s)", key, type)
+        log.debug("Missing property %s (%s)", key, etype)
         return None
     except (XError, PropertyError):
         if not ignore_errors:
-            log.info("Missing window or missing property or wrong property type %s (%s)", key, type)
-            import traceback
-            traceback.print_stack()
+            log.info("Missing window or missing property or wrong property type %s (%s)", key, etype)
+            traceback.print_exc()
         return None
     try:
-        return _prop_decode(target, type, data)
+        return _prop_decode(target, etype, data)
     except:
         log.warn("Error parsing property %s (type %s); this may be a"
                  + " misbehaving application, or bug in Wimpiggy\n"
                  + "  Data: %r[...?]",
-                 key, type, data[:160])
+                 key, etype, data[:160])
+        if not ignore_errors:
+            traceback.print_exc()
         raise
