@@ -673,6 +673,7 @@ class XpraClient(XpraClientBase):
         self.mmap_enabled = self.supports_mmap and self.mmap_file and capabilities.get("mmap_enabled")
         self.server_auto_refresh_delay = capabilities.get("auto_refresh_delay", 0)/1000
         self.change_quality = capabilities.get("change-quality", False)
+        self.xsettings_tuple = capabilities.get("xsettings-tuple", False)
         if self.mmap_enabled:
             log.info("mmap is enabled using %sBytes area in %s", std_unit(self.mmap_size), self.mmap_file)
         #the server will have a handle on the mmap file by now, safe to delete:
@@ -690,7 +691,7 @@ class XpraClient(XpraClientBase):
         self.emit("clipboard-toggled")
         self.key_repeat_delay, self.key_repeat_interval = capabilities.get("key_repeat", (-1,-1))
         self.emit("handshake-complete")
-        if self.clipboard_enabled:
+        if self.server_supports_clipboard:
             #from now on, we will send a message to the server whenever the clipboard flag changes:
             self.connect("clipboard-toggled", self.send_clipboard_enabled_status)
         if self.toggle_keyboard_sync:
@@ -823,17 +824,18 @@ class XpraClient(XpraClientBase):
         window = self._id_to_window.get(wid)
         if not window:
             #window is gone
-            if coding=="mmap":
-                assert self.mmap_enabled
-                def free_mmap_area():
-                    #we need to ack the data to free the space!
-                    data_start = ctypes.c_uint.from_buffer(self.mmap, 0)
-                    offset, length = data[-1]
-                    data_start.value = offset+length
-                #clear the mmap area via idle_add so any pending draw requests
-                #will get a chance to run first
-                gobject.idle_add(free_mmap_area)
-            self.send_damage_sequence(wid, packet_sequence, width, height, -1)
+            def draw_cleanup():
+                if coding=="mmap":
+                    assert self.mmap_enabled
+                    def free_mmap_area():
+                        #we need to ack the data to free the space!
+                        data_start = ctypes.c_uint.from_buffer(self.mmap, 0)
+                        offset, length = data[-1]
+                        data_start.value = offset+length
+                    #clear the mmap area via idle_add so any pending draw requests
+                    #will get a chance to run first (preserving the order)
+                self.send_damage_sequence(wid, packet_sequence, width, height, -1)
+            gobject.idle_add(draw_cleanup)
             return
         options = {}
         if len(packet)>10:
