@@ -4,7 +4,9 @@
 # Parti is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os
 import sys
+import hashlib
 from wimpiggy.gobject_compat import import_gobject
 gobject = import_gobject()
 
@@ -12,17 +14,23 @@ from wimpiggy.util import n_arg_signal
 from wimpiggy.log import Logger
 log = Logger()
 
-from xpra.protocol import Protocol
+from xpra.protocol import Protocol, has_rencode
 from xpra.scripts.main import ENCODINGS
-from xpra.version_util import is_compatible_with
-
-import xpra
+from xpra.version_util import is_compatible_with, add_version_info
+from xpra.platform import get_machine_id
 
 def nn(x):
     if x is None:
         return  ""
     return x
 
+EXIT_OK = 0
+EXIT_CONNECTION_LOST = 1
+EXIT_TIMEOUT = 2
+EXIT_PASSWORD_REQUIRED = 3
+EXIT_PASSWORD_FILE_ERROR = 4
+EXIT_INCOMPATIBLE_VERSION = 5
+
 class ClientSource(object):
     def __init__(self, protocol):
         self._priority_packets = []
@@ -72,27 +80,34 @@ class XpraClientBase(gobject.GObject):
 
     __gsignals__ = {
         "handshake-complete": n_arg_signal(0),
+        "first-ui-received" : n_arg_signal(0),
         "received-gibberish": n_arg_signal(1),
         }
 
     def __init__(self, opts):
         gobject.GObject.__init__(self)
         self.exit_code = None
+        self.compression_level = opts.compression_level
+        self.password = None
         self.password_file = opts.password_file
         self.encoding = opts.encoding
-        self.jpegquality = opts.jpegquality
+        self.quality = opts.quality
         self._protocol = None
         self.server_capabilities = {}
+        self._remote_version = None
+        self._remote_revision = None
         self.init_packet_handlers()
 
     def ready(self, conn):
         log.debug("ready(%s)", conn)
         self._protocol = Protocol(conn, self.process_packet)
+        self._protocol.set_compression_level(self.compression_level)
         ClientSource(self._protocol)
         self._protocol.start()
 
     def init_packet_handlers(self):
-        self._packet_handlers = {
+        self._packet_handlers = {}
+        self._ui_packet_handlers = {
             "challenge": self._process_challenge,
             "disconnect": self._process_disconnect,
             "hello": self._process_hello,
@@ -107,26 +122,43 @@ class XpraClientBase(gobject.GObject):
         self.send(["hello", hello])
 
     def make_hello(self, challenge_response=None):
-        capabilities = {"version": xpra.__version__}
+        capabilities = {}
+        add_version_info(capabilities)
         if challenge_response:
             capabilities["challenge_response"] = challenge_response
         if self.encoding:
             capabilities["encoding"] = self.encoding
         capabilities["encodings"] = ENCODINGS
-        if self.jpegquality:
-            capabilities["jpeg"] = self.jpegquality
+        if self.quality>=0:
+            capabilities["jpeg"] = self.quality
+            capabilities["quality"] = self.quality
         capabilities["platform"] = sys.platform
+        capabilities["client_type"] = "Python/Gobject"
         capabilities["raw_packets"] = True
+        capabilities["chunked_compression"] = True
+        capabilities["rencode"] = has_rencode
         capabilities["server-window-resize"] = True
+        u = hashlib.sha512()
+        u.update(str(get_machine_id()))
+        if os.name=="posix":
+            u.update("/")
+            u.update(str(os.getuid()))
+            u.update("/")
+            u.update(str(os.getgid()))
+        capabilities["uuid"] = u.hexdigest()
         capabilities["randr_notify"] = False    #only client.py cares about this
+        capabilities["windows"] = False         #only client.py cares about this
         return capabilities
 
+    def idle_send(self, packet):
+        gobject.idle_add(self.send, packet)
+
     def send(self, packet):
-        if self._protocol:
+        if self._protocol and self._protocol.source:
             self._protocol.source.queue_ordinary_packet(packet)
 
     def send_now(self, packet):
-        if self._protocol:
+        if self._protocol and self._protocol.source:
             self._protocol.source.queue_priority_packet(packet)
 
     def cleanup(self):
@@ -142,45 +174,61 @@ class XpraClientBase(gobject.GObject):
             self.exit_code = exit_code
         raise Exception("override me!")
 
+    def warn_and_quit(self, exit_code, warning):
+        log.warn(warning)
+        self.quit(exit_code)
+
     def _process_disconnect(self, packet):
-        log.error("server requested disconnect: %s", packet[1:])
-        self.quit(0)
+        if len(packet)==2:
+            info = packet[1]
+        else:
+            info = packet[1:]
+        self.warn_and_quit(EXIT_OK, "server requested disconnect: %s" % info)
 
     def _process_connection_lost(self, packet):
-        log.error("Connection lost")
-        self.quit(1)
+        self.warn_and_quit(EXIT_CONNECTION_LOST, "Connection lost")
 
     def _process_challenge(self, packet):
-        if not self.password_file:
-            log.error("password is required by the server")
-            self.quit(2)
+        if not self.password_file and not self.password:
+            self.warn_and_quit(EXIT_PASSWORD_REQUIRED, "password is required by the server")
             return
-        import hmac
+        if not self.password:
+            self.load_password()
+            log("password read from file %s is %s", self.password_file, self.password)
+        if self.password:
+            salt = packet[1]
+            import hmac
+            challenge_response = hmac.HMAC(self.password, salt)
+            self.send_hello(challenge_response.hexdigest())
+
+    def load_password(self):
         try:
             passwordFile = open(self.password_file, "rU")
-            password = passwordFile.read()
+            self.password = passwordFile.read()
+            passwordFile.close()
+            while self.password.endswith("\n") or self.password.endswith("\r"):
+                self.password = self.password[:-1]
         except IOError, e:
-            log.error("failed to open password file %s: %s", self.password_file, e)
-            self.quit(3)
-            return
-        salt = packet[1]
-        challenge_response = hmac.HMAC(password, salt)
-        self.send_hello(challenge_response.hexdigest())
+            self.warn_and_quit(EXIT_PASSWORD_FILE_ERROR, "failed to open password file %s: %s" % (self.password_file, e))
 
     def _process_hello(self, packet):
         self.server_capabilities = packet[1]
         self.parse_server_capabilities(self.server_capabilities)
 
     def parse_server_capabilities(self, capabilities):
-        self._protocol.raw_packets = bool(capabilities.get("raw_packets", False))
-        self._remote_version = capabilities.get("version") or capabilities.get("__prerelease_version")
+        self._remote_version = capabilities.get("version")
+        self._remote_revision = capabilities.get("revision")
         if not is_compatible_with(self._remote_version):
-            self.quit(4)
+            self.warn_and_quit(EXIT_INCOMPATIBLE_VERSION, "incompatible remote version: %s" % self._remote_version)
+            return False
+        if capabilities.get("rencode") and has_rencode:
+            self._protocol.enable_rencode()
+        self._protocol.chunked_compression = capabilities.get("chunked_compression", False)
+        return True
 
     def _process_set_deflate(self, packet):
-        #this tell us the server has set its compressor
-        #(the decompressor has been enabled - see protocol)
-        log.debug("set_deflate: %s", packet[1:])
+        #legacy, should not be used for anything
+        pass
 
     def _process_gibberish(self, packet):
         (_, data) = packet
@@ -190,10 +238,14 @@ class XpraClientBase(gobject.GObject):
     def process_packet(self, proto, packet):
         packet_type = packet[0]
         handler = self._packet_handlers.get(packet_type)
+        if handler:
+            handler(packet)
+            return
+        handler = self._ui_packet_handlers.get(packet_type)
         if not handler:
             log.error("unknown packet type: %s", packet_type)
             return
-        handler(packet)
+        gobject.idle_add(handler, packet)
 
 gobject.type_register(XpraClientBase)
 
@@ -240,6 +292,7 @@ class GLibXpraClient(XpraClientBase):
     def make_hello(self, challenge_response=None):
         capabilities = XpraClientBase.make_hello(self, challenge_response)
         capabilities["keyboard"] = False
+        capabilities["client_type"] = "Python/Glib"
         return capabilities
 
     def quit(self, exit_code):
@@ -258,24 +311,24 @@ class ScreenshotXpraClient(GLibXpraClient):
     def __init__(self, conn, opts, screenshot_filename):
         self.screenshot_filename = screenshot_filename
         def screenshot_timeout(*args):
-            self.exit_code = 1
-            log.error("timeout: did not receive the screenshot")
-            self.quit(5)
+            self.warn_and_quit(EXIT_TIMEOUT, "timeout: did not receive the screenshot")
         gobject.timeout_add(10*1000, screenshot_timeout)
         GLibXpraClient.__init__(self, conn, opts)
 
     def _process_screenshot(self, packet):
         (w, h, encoding, _, img_data) = packet[1:6]
         assert encoding=="png"
+        if len(img_data)==0:
+            self.warn_and_quit(EXIT_OK, "screenshot is empty and has not been saved (maybe there are no windows or they are not currently shown)")
+            return
         f = open(self.screenshot_filename, 'wb')
         f.write(img_data)
         f.close()
-        log.info("screenshot %sx%s saved to: %s", w, h, self.screenshot_filename)
-        self.quit(0)
+        self.warn_and_quit(EXIT_OK, "screenshot %sx%s saved to: %s" % (w, h, self.screenshot_filename))
 
     def init_packet_handlers(self):
         GLibXpraClient.init_packet_handlers(self)
-        self._packet_handlers["screenshot"] = self._process_screenshot
+        self._ui_packet_handlers["screenshot"] = self._process_screenshot
 
     def make_hello(self, challenge_response=None):
         capabilities = GLibXpraClient.make_hello(self, challenge_response)
@@ -290,9 +343,7 @@ class InfoXpraClient(GLibXpraClient):
 
     def __init__(self, conn, opts):
         def info_timeout(*args):
-            self.exit_code = 1
-            log.error("timeout: did not receive the info")
-            self.quit(5)
+            self.warn_and_quit(EXIT_TIMEOUT, "timeout: did not receive the info")
         gobject.timeout_add(10*1000, info_timeout)
         GLibXpraClient.__init__(self, conn, opts)
 
@@ -327,8 +378,7 @@ class VersionXpraClient(GLibXpraClient):
     def _process_hello(self, packet):
         log.debug("process_hello: %s", packet)
         props = packet[1]
-        log.info("%s" % props.get("version"))
-        self.quit(0)
+        self.warn_and_quit(EXIT_OK, str(props.get("version")))
 
     def make_hello(self, challenge_response=None):
         capabilities = GLibXpraClient.make_hello(self, challenge_response)
@@ -342,8 +392,7 @@ class StopXpraClient(GLibXpraClient):
 
     def __init__(self, conn, opts):
         def stop_timeout(*args):
-            log.error("timeout: server did not disconnect us")
-            self.quit(5)
+            self.warn_and_quit(EXIT_TIMEOUT, "timeout: server did not disconnect us")
         gobject.timeout_add(5*1000, stop_timeout)
         GLibXpraClient.__init__(self, conn, opts)
 
