@@ -1,5 +1,5 @@
 # This file is part of Parti.
-# Copyright (C) 2011 Antoine Martin <antoine@nagafix.co.uk>
+# Copyright (C) 2011, 2012 Antoine Martin <antoine@nagafix.co.uk>
 # Parti is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -14,3 +14,6 @@ GOT_PASSWORD_PROMPT_SUGGESTION = "Perhaps you need to set up your ssh agent?\n"
 
 def add_client_options(parser):
     pass
+
+def get_machine_id():
+    return  ""
