@@ -13,15 +13,16 @@ log = Logger()
 
 
 class ClientExtras(ClientExtrasBase):
-    def __init__(self, client, opts):
-        ClientExtrasBase.__init__(self, client, opts)
+    def __init__(self, client, opts, conn):
+        ClientExtrasBase.__init__(self, client, opts, conn)
         self.locate_icon_filename(opts.tray_icon)
-        self.setup_growl()
+        self.setup_growl(opts.notifications)
         self.setup_macdock()
-        self.clipboard_helper = None
 
-    def setup_growl(self):
+    def setup_growl(self, enabled):
         self.growl_notifier = None
+        if not enabled:
+            return
         try:
             import Growl        #@UnresolvedImport
             name = self.client.session_name or "Xpra"
@@ -73,18 +74,23 @@ class ClientExtras(ClientExtrasBase):
             self.macapp.set_menu_bar(self.menu_bar)
             self.quit_menu_item.hide()
 
-            self.macapp.insert_app_menu_item(self.make_aboutmenuitem(), 0)
-            self.macapp.insert_app_menu_item(self.make_sessioninfomenuitem(), 1)
-            self.macapp.insert_app_menu_item(self.make_bellmenuitem(), 2)
-            self.macapp.insert_app_menu_item(self.make_notificationsmenuitem(), 3)
-            self.macapp.insert_app_menu_item(self.make_encodingsmenuitem(), 4)
+            self._menu_item_pos = 0
+            def add_item(item):
+                self.macapp.insert_app_menu_item(item, self._menu_item_pos)
+                self._menu_item_pos += 1
+            add_item(self.make_aboutmenuitem())
+            add_item(self.make_sessioninfomenuitem())
+            add_item(self.make_bellmenuitem())
+            add_item(self.make_cursorsmenuitem())
+            add_item(self.make_notificationsmenuitem())
+            add_item(self.make_encodingsmenuitem())
             if not self.client.readonly:
-                self.macapp.insert_app_menu_item(self.make_layoutsmenuitem(), 5)
-            self.macapp.insert_app_menu_item(self.make_jpegsubmenu(), 6)
-            self.macapp.insert_app_menu_item(self.make_compressionmenu(), 7)
-            self.macapp.insert_app_menu_item(self.make_refreshmenuitem(), 8)
-            self.macapp.insert_app_menu_item(self.make_raisewindowsmenuitem(), 9)
-            self.macapp.insert_app_menu_item(gtk.SeparatorMenuItem(), 10)
+                add_item(self.make_layoutsmenuitem())
+            add_item(self.make_qualitysubmenu())
+            #add_item(self.make_compressionmenu())
+            add_item(self.make_refreshmenuitem())
+            add_item(self.make_raisewindowsmenuitem())
+            add_item(gtk.SeparatorMenuItem())
 
             #dock menu
             self.dock_menu = gtk.Menu()
@@ -112,7 +118,7 @@ class ClientExtras(ClientExtrasBase):
     def can_notify(self):
         return  self.growl_notifier is not None
 
-    def show_notify(self, dbus_id, id, app_name, replaces_id, app_icon, summary, body, expire_timeout):
+    def show_notify(self, dbus_id, nid, app_name, replaces_id, app_icon, summary, body, expire_timeout):
         if not self.growl_notifier:
             return
         if self.icon_filename:
@@ -128,7 +134,7 @@ class ClientExtras(ClientExtrasBase):
         Carbon.Snd.SysBeep(1)
 
     def get_gtk_keymap(self):
-        return  get_gtk_keymap(add_if_missing=["Control_L"])
+        return  get_gtk_keymap()
 
     def grok_modifier_map(self, display_source, xkbmap_mod_meanings):
         modifiers = ClientExtrasBase.grok_modifier_map(self, display_source, xkbmap_mod_meanings)
