@@ -19,33 +19,34 @@ We register this class as handling notifications on the session dbus,
 optionally replacing an existing instance if one exists.
 
 The generalized callback signatures are:
- notify_callback(dbus_id, id, app_name, replaces_id, app_icon, summary, body, expire_timeout)
- close_callback(id)
+ notify_callback(dbus_id, nid, app_name, replaces_nid, app_icon, summary, body, expire_timeout)
+ close_callback(nid)
 """
 class DBUSNotificationsForwarder(dbus.service.Object):
 
     CAPABILITIES = ["body", "icon-static"]
 
     def __init__(self, bus, notify_callback=None, close_callback=None):
+        self.bus = bus
         self.notify_callback = notify_callback
         self.close_callback = close_callback
         self.counter = 0
         self.dbus_id = os.environ.get("DBUS_SESSION_BUS_ADDRESS", "")
         bus_name = dbus.service.BusName(BUS_NAME, bus=bus)
         dbus.service.Object.__init__(self, bus_name, BUS_PATH)
- 
+
     @dbus.service.method(BUS_NAME, in_signature='susssasa{sv}i', out_signature='u')
-    def Notify(self, app_name, replaces_id, app_icon, summary, body, actions, hints, expire_timeout):
-        log("Notify(%s,%s,%s,%s,%s,%s,%s,%s)" % (app_name, replaces_id, app_icon, summary, body, actions, hints, expire_timeout))
-        if replaces_id==0:
+    def Notify(self, app_name, replaces_nid, app_icon, summary, body, actions, hints, expire_timeout):
+        log("Notify(%s,%s,%s,%s,%s,%s,%s,%s)" % (app_name, replaces_nid, app_icon, summary, body, actions, hints, expire_timeout))
+        if replaces_nid==0:
             self.counter += 1
-            id = self.counter
+            nid = self.counter
         else:
-            id = replaces_id
+            nid = replaces_nid
         if self.notify_callback:
-            self.notify_callback(self.dbus_id, id, app_name, replaces_id, app_icon, summary, body, expire_timeout)
-        log("Notify returning %s", id)
-        return id
+            self.notify_callback(self.dbus_id, nid, app_name, replaces_nid, app_icon, summary, body, expire_timeout)
+        log("Notify returning %s", nid)
+        return nid
 
     @dbus.service.method(BUS_NAME, out_signature='ssss')
     def GetServerInformation(self):
@@ -59,17 +60,27 @@ class DBUSNotificationsForwarder(dbus.service.Object):
         return DBUSNotificationsForwarder.CAPABILITIES
 
     @dbus.service.method(BUS_NAME, in_signature='u')
-    def CloseNotification(self, id):
-        log("CloseNotification(%s)", id)
+    def CloseNotification(self, nid):
+        log("CloseNotification(%s)", nid)
         if self.close_callback:
-            self.close_callback(id)
+            self.close_callback(nid)
+
+    def release(self):
+        self.bus.release_name(BUS_NAME)
+
+    def __str__(self):
+        return  "DBUSNotificationsForwarder(%s)" % BUS_NAME
 
 def register(notify_callback=None, close_callback=None, replace=False):
     DBusGMainLoop(set_as_default=True)
     bus = dbus.SessionBus()
+    flags = dbus.bus.NAME_FLAG_DO_NOT_QUEUE
     if replace:
-        request = bus.request_name(BUS_NAME, dbus.bus.NAME_FLAG_REPLACE_EXISTING)
-        log("request_name(%s)=%s" % (BUS_NAME, request))
+        flags |= dbus.bus.NAME_FLAG_REPLACE_EXISTING
+    request = bus.request_name(BUS_NAME, flags)
+    if request==dbus.bus.REQUEST_NAME_REPLY_EXISTS:
+        raise Exception("the name '%s' is already claimed on the session bus" % BUS_NAME)
+    log("notifications: bus name '%s', request=%s" % (BUS_NAME, request))
     return DBUSNotificationsForwarder(bus, notify_callback, close_callback)
 
 def main():
