@@ -1,8 +1,10 @@
 # This file is part of Parti.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
+# Copyright (C) 2012 Antoine Martin <antoine@devloop.org.uk>
 # Parti is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import os
 import pygtk
 pygtk.require("2.0")
 import struct
@@ -14,6 +16,7 @@ from wimpiggy.log import Logger
 log = Logger()
 
 from xpra.nested_main import NestedMainLoop
+from xpra.protocol import zlib_compress
 
 #debug = log.info
 debug = log.debug
@@ -21,6 +24,7 @@ debug = log.debug
 class ClipboardProtocolHelperBase(object):
     def __init__(self, send_packet_cb, clipboards=["CLIPBOARD"]):
         self.send = send_packet_cb
+        self.max_clipboard_packet_size = 32*1024 - 1024
         self._clipboard_proxies = {}
         for clipboard in clipboards:
             proxy = ClipboardProxy(clipboard)
@@ -62,11 +66,11 @@ class ClipboardProtocolHelperBase(object):
         del self._clipboard_outstanding_requests[request_id]
         return result
 
-    def _clipboard_got_contents(self, request_id, type, format, data):
-        debug("got clipboard contents(%s)=%s (type=%s, format=%s)", request_id, len(data or []), type, format)
+    def _clipboard_got_contents(self, request_id, dtype, dformat, data):
+        debug("got clipboard contents(%s)=%s (type=%s, format=%s)", request_id, len(data or []), dtype, dformat)
         if request_id in self._clipboard_outstanding_requests:
             loop = self._clipboard_outstanding_requests[request_id]
-            loop.done({"type": type, "format": format, "data": data})
+            loop.done({"type": dtype, "format": dformat, "data": data})
         else:
             debug("got unexpected response to clipboard request %s", request_id)
 
@@ -74,93 +78,103 @@ class ClipboardProtocolHelperBase(object):
         debug("send clipboard token: %s", selection)
         self.send(["clipboard-token", self.local_to_remote(selection)])
 
-    def _munge_raw_selection_to_wire(self, target, type, format, data):
+    def _munge_raw_selection_to_wire(self, target, dtype, dformat, data):
         # Some types just cannot be marshalled:
         if type in ("WINDOW", "PIXMAP", "BITMAP", "DRAWABLE",
                     "PIXEL", "COLORMAP"):
-            debug("skipping clipboard data of type: %s, format=%s, len(data)=%s", type, format, len(data))
-            return (None, None)
-        return self._do_munge_raw_selection_to_wire(target, type, format, data)
+            debug("skipping clipboard data of type: %s, format=%s, len(data)=%s", dtype, dformat, len(data))
+            return None, None
+        return self._do_munge_raw_selection_to_wire(target, dtype, dformat, data)
 
-    def _do_munge_raw_selection_to_wire(self, target, type, format, data):
+    def _do_munge_raw_selection_to_wire(self, target, dtype, dformat, data):
         """ this method is overriden in xclipboard to parse X11 atoms """
         # Other types need special handling, and all types need to be
         # converting into an endian-neutral format:
-        if format == 32:
-            if type in ("ATOM", "ATOM_PAIR"):
-                #we cannot handle gdk atoms here (but xclipboard does)
+        debug("_do_munge_raw_selection_to_wire(%s, %s, %s, %s:%s)", target, dtype, dformat, type(data), len(data or ""))
+        if dformat == 32:
+            #you should be using gdk_clipboard for atom support!
+            if dtype in ("ATOM", "ATOM_PAIR") and os.name=="posix":
+                #we cannot handle gdk atoms here (but gdk_clipboard does)
                 return None, None
-            #important note: on 64 bits, format=32 means 8 bytes, not 4 
-            #that's just the way it is... 
-            sizeof_long = struct.calcsize("@L")
-            assert sizeof_long in (4, 8), "struct.calcsize('@L)=%s" % sizeof_long
+            #important note: on 64 bits, format=32 means 8 bytes, not 4
+            #that's just the way it is...
+            sizeof_long = struct.calcsize('@L')
+            assert sizeof_long in (4, 8), "struct.calcsize('@L')=%s" % sizeof_long
             binfmt = "@" + "L" * (len(data) // sizeof_long)
             ints = struct.unpack(binfmt, data)
-            return ("integers", ints)
-        elif format == 16:
-            sizeof_short = struct.calcsize("=H")
-            assert sizeof_short == 2
-            binfmt = "@" + "H" * (len(data) // sizeof_short)
+            return "integers", ints
+        elif dformat == 16:
+            sizeof_short = struct.calcsize('=H')
+            assert sizeof_short == 2, "struct.calcsize('=H')=%s" % sizeof_short
+            binfmt = "=" + "H" * (len(data) // sizeof_short)
             ints = struct.unpack(binfmt, data)
-            return ("integers", ints)
-        elif format == 8:
-            return ("bytes", data)
+            return "integers", ints
+        elif dformat == 8:
+            return "bytes", data
         else:
-            log.error("unhandled format %s for clipboard data type %s" % (format, type))
-            return (None, None)
+            log.error("unhandled format %s for clipboard data type %s" % (dformat, dtype))
+            return None, None
 
-    def _munge_wire_selection_to_raw(self, encoding, type, format, data):
-        debug("wire selection to raw, encoding=%s, type=%s, format=%s, len(data)=%s", encoding, type, format, len(data))
+    def _munge_wire_selection_to_raw(self, encoding, dtype, dformat, data):
+        debug("wire selection to raw, encoding=%s, type=%s, format=%s, len(data)=%s", encoding, dtype, dformat, len(data))
         if encoding == "bytes":
             return data
         elif encoding == "integers":
-            if format == 32:
+            if dformat == 32:
                 format_char = "L"
-            elif format == 16:
+            elif dformat == 16:
                 format_char = "H"
-            elif format == 8:
+            elif dformat == 8:
                 format_char = "B"
             else:
-                raise Exception("unknown encoding format: %s" % format)
-            return struct.pack("@" + format_char * len(data), *data)
+                raise Exception("unknown encoding format: %s" % dformat)
+            fstr = "@" + format_char * len(data)
+            debug("struct.pack(%s, %s)", fstr, data)
+            return struct.pack(fstr, *data)
         else:
-            assert False
+            raise Exception("unhanled encoding: %s" % encoding)
 
     def _process_clipboard_request(self, packet):
-        (_, request_id, selection, target) = packet
+        request_id, selection, target = packet[1:4]
         name = self.remote_to_local(selection)
         debug("process clipboard request, request_id=%s, selection=%s, local name=%s, target=%s", request_id, selection, name, target)
         if name in self._clipboard_proxies:
             proxy = self._clipboard_proxies[name]
-            def got_contents(type, format, data):
-                debug("got_contents(%s,%s,%s)", type, format, len(data or []))
+            def got_contents(dtype, dformat, data):
+                debug("got_contents(%s, %s, %s:%s) data=%s, str(data)=%s", dtype, dformat, type(data), len(data or ""), list(bytearray(data or "")[:200]), str(data)[:200])
                 def no_contents():
                     self.send(["clipboard-contents-none", request_id, selection])
-                if type is None or data is None:
+                if dtype is None or data is None:
                     no_contents()
                     return
-                munged = self._munge_raw_selection_to_wire(target, type, format, data)
-                (wire_encoding, wire_data) = munged
-                log("clipboard raw -> wire: %r -> %r", (type, format, data), munged)
+                munged = self._munge_raw_selection_to_wire(target, dtype, dformat, data)
+                wire_encoding, wire_data = munged
+                debug("clipboard raw -> wire: %r -> %r", (dtype, dformat, data), munged)
                 if wire_encoding is None:
                     no_contents()
                     return
+                if len(wire_data)>256:
+                    wire_data = zlib_compress("clipboard: %s / %s" % (dtype, dformat), wire_data)
+                    if len(wire_data)>self.max_clipboard_packet_size:
+                        log.warn("even compressed, clipboard contents are too big and have not been sent: %s compressed bytes dropped" % len(wire_data))
+                        no_contents()
+                        return
                 self.send(["clipboard-contents", request_id, selection,
-                           type, format, wire_encoding, wire_data])
+                           dtype, dformat, wire_encoding, wire_data])
             proxy.get_contents(target, got_contents)
         else:
             self.send(["clipboard-contents-none", request_id, selection])
 
     def _process_clipboard_contents(self, packet):
-        (_, request_id, selection, type, format, wire_encoding, wire_data) = packet
-        debug("process clipboard contents, selection=%s, type=%s, format=%s", selection, type, format)
-        raw_data = self._munge_wire_selection_to_raw(wire_encoding, type, format, wire_data)
-        debug("clipboard wire -> raw: %r -> %r", (type, format, wire_encoding, wire_data), raw_data)
-        self._clipboard_got_contents(request_id, type, format, raw_data)
+        request_id, selection, dtype, dformat, wire_encoding, wire_data = packet[1:8]
+        debug("process clipboard contents, selection=%s, type=%s, format=%s", selection, dtype, dformat)
+        raw_data = self._munge_wire_selection_to_raw(wire_encoding, dtype, dformat, wire_data)
+        debug("clipboard wire -> raw: %r -> %r", (dtype, dformat, wire_encoding, wire_data), raw_data)
+        self._clipboard_got_contents(request_id, dtype, dformat, raw_data)
 
     def _process_clipboard_contents_none(self, packet):
         debug("process clipboard contents none")
-        (_, request_id, _) = packet
+        request_id = packet[1]
         self._clipboard_got_contents(request_id, None, None, None)
 
     _packet_handlers = {
@@ -176,6 +190,16 @@ class ClipboardProtocolHelperBase(object):
         self._packet_handlers[packet_type](self, packet)
 
 
+class DefaultClipboardProtocolHelper(ClipboardProtocolHelperBase):
+    """
+        Default clipboard implementation with all 3 selections.
+        But without gdk atom support, see gdk_clipboard for a better one!
+    """
+
+    def __init__(self, send_packet_cb):
+        ClipboardProtocolHelperBase.__init__(self, send_packet_cb, ["CLIPBOARD", "PRIMARY", "SECONDARY"])
+
+
 class ClipboardProxy(gtk.Invisible):
     __gsignals__ = {
         # arguments: (selection, target)
@@ -235,9 +259,9 @@ class ClipboardProxy(gtk.Invisible):
                 from wimpiggy.prop import prop_get
                 from wimpiggy.error import trap
             except ImportError:
+                debug("MULTIPLE for property '%s' not handled due to missing wimpiggy bindings", event.property)
                 gtk.Invisible.do_selection_request_event(self, event)
                 return
-
             def get_targets(targets):
                 atoms = prop_get(event.window, event.property, ["multiple-conversion"])
                 debug("MULTIPLE clipboard atoms: %r", atoms)
@@ -248,7 +272,9 @@ class ClipboardProxy(gtk.Invisible):
             for target in targets:
                 self.selection_add_target(self._selection, target, 0)
         else:
+            debug("target for %s: %r", self._selection, target)
             self.selection_add_target(self._selection, target, 0)
+        debug("do_selection_request_event(%s) target=%s, selection=%s", event, target, self._selection)
         gtk.Invisible.do_selection_request_event(self, event)
 
     # This function is called by GTK+ when we own the clipboard and a local
@@ -257,13 +283,16 @@ class ClipboardProxy(gtk.Invisible):
         # Either call selection_data.set() or don't, and then return.
         # In practice, send a call across the wire, then block in a recursive
         # main loop.
-        debug("do_selection_get(%s,%s,%s)", selection_data, info, time)
+        debug("do_selection_get(%s, %s, %s)", selection_data, info, time)
         assert self._selection == str(selection_data.selection)
         target = str(selection_data.target)
         result = self.emit("get-clipboard-from-remote", self._selection, target)
         if result is not None and result["type"] is not None:
-            debug("do_selection_get(%s,%s,%s) calling selection_data.set(%s,%s,%s)", selection_data, info, time, result["type"], result["format"], len(result["data"]))
-            selection_data.set(result["type"], result["format"], result["data"])
+            data = result["data"]
+            dformat = result["format"]
+            dtype = result["type"]
+            debug("do_selection_get(%s,%s,%s) calling selection_data.set(%s, %s, %s:%s)", selection_data, info, time, dtype, dformat, type(data), len(data or ""))
+            selection_data.set(dtype, dformat, data)
         else:
             debug("remote selection fetch timed out")
 
@@ -280,7 +309,7 @@ class ClipboardProxy(gtk.Invisible):
 
     def got_token(self):
         # We got the anti-token.
-        debug("got token")
+        debug("got token, selection=%s", self._selection)
         self._have_token = True
         if not self.selection_owner_set(self._selection):
             # I don't know how this can actually fail, given that we pass
@@ -299,9 +328,12 @@ class ClipboardProxy(gtk.Invisible):
                      + "*I* thought *they* had it... weird.")
             cb(None, None, None)
         def unpack(clipboard, selection_data, data):
+            debug("unpack(%s, %s, %s:%s)", clipboard, selection_data, type(data), len(data or ""))
             if selection_data is None:
                 cb(None, None, None)
             else:
+                debug("unpack(..) type=%s, format=%s, data=%s:%s", selection_data.type, selection_data.format,
+                            type(selection_data.data), len(selection_data.data or ""))
                 cb(str(selection_data.type),
                    selection_data.format,
                    selection_data.data)
