@@ -348,7 +348,8 @@ class XpraServer(gobject.GObject):
                 self.notifications_forwarder.release()
             except Exception, e:
                 log.error("failed to release dbus notification forwarder: %s", e)
-        for proto in self._server_sources.keys():
+        log("cleanup will disconnect: %s", self._potential_protocols)
+        for proto in self._potential_protocols:
             self.disconnect(proto, "shutting down")
 
     def _new_connection(self, listener, *args):
@@ -417,6 +418,7 @@ class XpraServer(gobject.GObject):
             ss.bell(wid, event.device, event.percent, event.pitch, event.duration, event.bell_class, event.bell_id, event.bell_name or "")
 
     def send_clipboard_packet(self, packet):
+        assert self._clipboard_helper is not None
         if self._clipboard_client:
             self._clipboard_client.send(packet)
 
@@ -493,8 +495,8 @@ class XpraServer(gobject.GObject):
                     ("min_size", "minimum-size"),
                     ("base_size", "base-size"),
                     ("resize_inc", "increment"),
-                    ("min_aspect_ratio", "minimum-aspect"),
-                    ("max_aspect_ratio", "maximum-aspect"),
+                    ("min_aspect_ratio", "minimum-aspect-ratio"),
+                    ("max_aspect_ratio", "maximum-aspect-ratio"),
                     ]:
                     v = getattr(hints, attr)
                     if v is not None:
@@ -793,10 +795,23 @@ class XpraServer(gobject.GObject):
         if proto._closed:
             return
         def force_disconnect(*args):
+            self.cleanup_source(proto)
             proto.close()
         proto._add_packet_to_queue(["disconnect", reason])
         gobject.timeout_add(1000, force_disconnect)
 
+    def cleanup_source(self, protocol):
+        #this ensures that from now on we ignore any incoming packets coming
+        #from this connection as these could potentially set some keys pressed, etc
+        source = self._server_sources.get(protocol)
+        if source:
+            source.close()
+            del self._server_sources[protocol]
+            log.info("xpra client disconnected.")
+        if protocol in self._potential_protocols:
+            self._potential_protocols.remove(protocol)
+        return source
+
     def _send_password_challenge(self, proto):
         proto.salt = "%s" % uuid.uuid4()
         log.info("Password required, sending challenge")
@@ -953,7 +968,8 @@ class XpraServer(gobject.GObject):
         else:
             root_w, root_h = gtk.gdk.get_default_root_window().get_size()
         #take the clipboard if no-one else has yet:
-        if ss.clipboard_enabled and (self._clipboard_client is None or self._clipboard_client.closed):
+        if ss.clipboard_enabled and self._clipboard_helper is not None and \
+            (self._clipboard_client is None or self._clipboard_client.closed):
             self._clipboard_client = ss
         #so only activate this feature afterwards:
         self.keyboard_sync = bool(capabilities.get("keyboard_sync", True))
@@ -1025,7 +1041,7 @@ class XpraServer(gobject.GObject):
         capabilities["toggle_cursors_bell_notify"] = True
         capabilities["toggle_keyboard_sync"] = True
         capabilities["notifications"] = self.notifications_forwarder is not None
-        capabilities["clipboard"] = self._clipboard_client == server_source
+        capabilities["clipboard"] = self._clipboard_helper is not None and self._clipboard_client == server_source
         capabilities["bell"] = self.bell
         capabilities["cursors"] = self.cursors
         if "key_repeat" in client_capabilities:
@@ -1124,12 +1140,7 @@ class XpraServer(gobject.GObject):
             log.info("Disconnecting existing client %s, reason is: %s", protocol, reason)
             # send message asking client to disconnect (politely):
             protocol.flush_then_close(["disconnect", reason])
-            #this ensures that from now on we ignore any incoming packets coming
-            #from this connection as these could potentially set some keys pressed, etc
-            ss = self._server_sources.get(protocol)
-            if ss:
-                ss.close()
-                del self._server_sources[protocol]
+            self.cleanup_source(protocol)
         #so it is now safe to clear them:
         #(this may fail during shutdown - which is ok)
         try:
@@ -1149,7 +1160,7 @@ class XpraServer(gobject.GObject):
             assert self._clipboard_client==self._server_sources.get(proto), \
                     "the request to change the clipboard enabled status does not come from the clipboard owner!"
             self._clipboard_client.clipboard_enabled = clipboard_enabled
-            log("toggled clipboard to %s", self.clipboard_enabled)
+            log("toggled clipboard to %s", clipboard_enabled)
         else:
             log.warn("client toggled clipboard-enabled but we do not support clipboard at all! ignoring it")
 
@@ -1481,15 +1492,9 @@ class XpraServer(gobject.GObject):
 
     def _process_connection_lost(self, proto, packet):
         log.info("Connection lost")
-        if proto in self._potential_protocols:
-            self._potential_protocols.remove(proto)
         if self._clipboard_client and self._clipboard_client.protocol==proto:
             self._clipboard_client = None
-        source = self._server_sources.get(proto)
-        if source:
-            del self._server_sources[proto]
-            source.close()
-            log.info("xpra client disconnected.")
+        source = self.cleanup_source(proto)
         if len(self._server_sources)==0:
             self._clear_keys_pressed()
             self._focus(source, 0, [])
@@ -1552,6 +1557,7 @@ class XpraServer(gobject.GObject):
             assert self._clipboard_client==ss, \
                     "the clipboard packet '%s' does not come from the clipboard owner!" % packet_type
             assert ss.clipboard_enabled, "received a clipboard packet from a source which does not have clipboard enabled!"
+            assert self._clipboard_helper, "received a clipboard packet but we do not support clipboard sharing"
             gobject.idle_add(self._clipboard_helper.process_clipboard_packet, packet)
             return
         if proto in self._server_sources:
