@@ -492,7 +492,7 @@ class ServerSource(object):
         if q>=0:
             self.default_damage_options["quality"] = q
         self.png_window_icons = "png" in self.encodings and "png" in ENCODINGS
-        self.auto_refresh_delay = int(capabilities.get("auto_refresh_delay", 0)/1000)
+        self.auto_refresh_delay = int(capabilities.get("auto_refresh_delay", 0))/1000.0
         #keyboard:
         self.keyboard_config = KeyboardConfig()
         self.keyboard_config.enabled = self.send_windows and bool(capabilities.get("keyboard", True))
@@ -649,6 +649,7 @@ class ServerSource(object):
         info["cursors%" % suffix] = self.send_cursors
         info["bell%" % suffix] = self.send_bell
         info["notifications%" % suffix] = self.send_notifications
+        info["auto_refresh%s" % suffix] = int(self.auto_refresh_delay*1000.0)
 
     def send_clipboard(self, packet):
         if self.clipboard_enabled:
@@ -786,7 +787,8 @@ class ServerSource(object):
                     for _, pixels, _, encoding_time in list(ws.statistics.encoding_stats):
                         total_pixels += pixels
                         total_time += encoding_time
-                    info["pixels_encoded_per_second%s" % suffix] = int(total_pixels / total_time)
+                    if total_time>0:
+                        info["pixels_encoded_per_second%s" % suffix] = int(total_pixels / total_time)
                     batch = ws.batch_config
                     for _,d in list(batch.last_delays):
                         batch_delays.append(d)
@@ -868,10 +870,12 @@ class ServerSource(object):
         """
         now = time.time()
         self.statistics.damage_packet_qsizes.append((now, len(self.damage_packet_queue)))
-        self.statistics.damage_packet_qpixels.append((now, wid, sum([x[1] for x in list(self.damage_packet_queue) if x[2]==wid])))
+        self.statistics.damage_packet_qpixels.append((now, wid, sum([x[2] for x in list(self.damage_packet_queue) if x[1]==wid])))
         self.damage_packet_queue.append((packet, wid, pixels, start_send_cb, end_send_cb))
         #if self.protocol._write_queue.empty():
-        gobject.idle_add(self.protocol.source_has_more)
+        p = self.protocol
+        if p:
+            gobject.idle_add(p.source_has_more)
 
 #
 # The damage packet thread loop:
