@@ -28,8 +28,6 @@
 #include "vpx/vp8dx.h"
 #include "vpxlib.h"
 #include "vpx/vpx_image.h"
-#define enc_interface (vpx_codec_vp8_cx())
-#define dec_interface (vpx_codec_vp8_dx())
 #define fourcc    0x30385056
 #define IVF_FILE_HDR_SZ  (32)
 #include <libswscale/swscale.h>
@@ -55,13 +53,14 @@ struct vpx_context *init_encoder(int width, int height)
 {
 	vpx_codec_enc_cfg_t  cfg;
 	struct vpx_context *ctx;
-	if (vpx_codec_enc_config_default(enc_interface, &cfg, 0))
+	vpx_codec_iface_t *codec_iface = vpx_codec_vp8_cx();
+	if (vpx_codec_enc_config_default(codec_iface, &cfg, 0))
 		return	NULL;
 	cfg.rc_target_bitrate = width * height * cfg.rc_target_bitrate / cfg.g_w / cfg.g_h;
 	cfg.g_w = width;
 	cfg.g_h = height;
 	ctx = malloc(sizeof(struct vpx_context));
-	if (vpx_codec_enc_init(&ctx->codec, enc_interface, &cfg, 0)) {
+	if (vpx_codec_enc_init(&ctx->codec, codec_iface, &cfg, 0)) {
 		codec_error(&ctx->codec, "vpx_codec_enc_init");
 		free(ctx);
 		return NULL;
@@ -81,9 +80,9 @@ void clean_encoder(struct vpx_context *ctx)
 struct vpx_context *init_decoder(int width, int height)
 {
 	struct vpx_context *ctx = malloc(sizeof(struct vpx_context));
+	vpx_codec_iface_t *codec_iface = vpx_codec_vp8_dx();
 	int              flags = 0;
-	//printf("Using %s\n", vpx_codec_iface_name(dec_interface));
-	int i = vpx_codec_dec_init(&ctx->codec, dec_interface, NULL, flags);
+	int i = vpx_codec_dec_init(&ctx->codec, codec_iface, NULL, flags);
 	if (i) {
 		codec_error(&ctx->codec, "vpx_codec_dec_init");
 		printf("vpx_codec_dec_init(..) failed with error %d\n", i);
@@ -149,7 +148,7 @@ int compress_image(struct vpx_context *ctx, vpx_image_t *image, uint8_t **out, i
 
 int csc_image_yuv2rgb(struct vpx_context *ctx, uint8_t *in[3], const int stride[3], uint8_t **out, int *outsz, int *outstride)
 {
-	uint8_t *dst[4] = { malloc(ctx->height * ctx->width * 3), NULL, NULL, NULL };
+	uint8_t *dst[4] = { xmemalign(ctx->height * ctx->width * 3), NULL, NULL, NULL };
 	int dststride[4] = { ctx->width * 3, 0, 0, 0 };
 
 	if (!ctx->yuv2rgb)
@@ -192,3 +191,34 @@ int decompress_image(struct vpx_context *ctx, uint8_t *in, int size, uint8_t *(*
 
 	return 0;
 }
+
+
+void* xmemalign(size_t size)
+{
+#ifdef MEMALIGN
+#ifdef _WIN32
+	//_aligned_malloc and _aligned_free lead to a memleak
+	//well done Microsoft, I didn't think you could screw up this badly
+	//and thank you for wasting my time once again
+	return malloc(size);
+#elif defined(__APPLE__) || defined(__OSX__)
+	//Crapple version: "all memory allocations are 16-byte aligned"
+	//no choice, this is what you get
+	return malloc(size);
+#else
+	//not WIN32 and not APPLE/OSX, assume POSIX:
+	void* memptr=NULL;
+	if (posix_memalign(&memptr, MEMALIGN_ALIGNMENT, size))
+		return	NULL;
+	return	memptr;
+#endif
+//MEMALIGN not set:
+#else
+	return	malloc(size);
+#endif
+}
+
+void xmemfree(void *ptr)
+{
+	free(ptr);
+}
