@@ -60,7 +60,7 @@ class DamageBatchConfig(object):
     MAX_PIXELS = 1024*1024*MAX_EVENTS   #small screen at MAX_EVENTS frames
     TIME_UNIT = 1                       #per second
     MIN_DELAY = 5                       #lower than 5 milliseconds does not make sense, just don't batch
-    START_DELAY = 100
+    START_DELAY = 50
     MAX_DELAY = 15000
     RECALCULATE_DELAY = 0.04            #re-compute delay 25 times per second at most
                                         #(this theoretical limit is never achieved since calculations take time + scheduling also does)
@@ -325,15 +325,17 @@ class WindowSource(object):
         if len(self.batch_config.last_actual_delays)>0:
             batch_delays = [x for _,x in list(self.batch_config.last_delays)]
             add_list_stats(info, "batch_delay"+suffix, batch_delays)
-        if self._video_encoder is not None:
-            try:
-                self._video_encoder_lock.acquire()
+        try:
+            quality_list, speed_list = None, None
+            self._video_encoder_lock.acquire()
+            if self._video_encoder is not None:
                 quality_list = [x for _, x in list(self._video_encoder_quality)]
-                add_list_stats(info, self._video_encoder.get_type()+"_quality"+suffix, quality_list, show_percentile=False)
                 speed_list = [x for _, x in list(self._video_encoder_speed)]
-                add_list_stats(info, self._video_encoder.get_type()+"_speed"+suffix, speed_list, show_percentile=False)
-            finally:
-                self._video_encoder_lock.release()
+        finally:
+            self._video_encoder_lock.release()
+        if quality_list and speed_list:
+            add_list_stats(info, self._video_encoder.get_type()+"_quality"+suffix, quality_list, show_percentile=False)
+            add_list_stats(info, self._video_encoder.get_type()+"_speed"+suffix, speed_list, show_percentile=False)
 
 
     def may_calculate_batch_delay(self, window):
@@ -549,7 +551,7 @@ class WindowSource(object):
             #x264 needs sizes divisible by 2:
             ww = ww & 0xFFFE
             wh = wh & 0xFFFE
-        if self.uses_swscale and (ww<8 or wh<=2):
+        if ww<8 or wh<=2:
             return switch()
         if pixel_count<ww*wh*0.01:
             #less than one percent of total area
@@ -606,10 +608,13 @@ class WindowSource(object):
             packet = self.make_data_packet(*data)
             if packet:
                 self.queue_damage_packet(pixmap, packet, damage_time, process_damage_time)
+                if self.encoding in ("png", "rgb24"):
+                    #primary encoding is lossless, no need for auto-refresh
+                    return
                 #auto-refresh:
                 if self.auto_refresh_delay>0 and not self.is_cancelled(sequence):
                     client_options = packet[10]     #info about this packet from the encoder
-                    gobject.idle_add(self.schedule_auto_refresh, window, w, h, self.encoding, options, client_options)
+                    gobject.idle_add(self.schedule_auto_refresh, window, w, h, coding, options, client_options)
         self.queue_damage(make_data_packet)
 
     def schedule_auto_refresh(self, window, w, h, coding, damage_options, client_options):
@@ -647,7 +652,7 @@ class WindowSource(object):
         if self._damage_delayed:
             #there is already a new damage region pending, let it re-schedule when it gets sent
             return
-        delay = max(self.auto_refresh_delay, int(1000*self.batch_config.delay))
+        delay = int(max(50, self.auto_refresh_delay, self.batch_config.delay))
         log("schedule_auto_refresh: low quality (%s%%) with %s pixels, (re)scheduling auto refresh timer with delay %s", actual_quality, w*h, delay)
         self.refresh_timer = gobject.timeout_add(delay, full_quality_refresh)
 
